@@ -1,13 +1,4 @@
-use std::{
-	io::{
-		Cursor,
-		Write,
-	},
-};
-
-use crate::tlv;
-
-use thiserror::Error;
+use crate::prelude::*;
 
 const MAGIC_NUMBER: [u8; 4] = [0x9a, 0x4a, 0x43, 0x81];
 pub const PACKET_SIZE: usize = 1024;
@@ -19,21 +10,33 @@ pub enum Message {
 	// 1
 	Request1 {
 		idem_id: [u8; 8],
-		mac: Option <[u8; 6]>
+		mac: Option <Mac>
 	},
 	// 2
-	Response1 (Option <[u8; 6]>),
+	Response1 (Option <Mac>),
 	// 3
 	Response2 (Response2),
 }
 
+impl Message {
+	pub fn new_request1 () -> Message {
+		let mut idem_id = [0u8; 8];
+		rand::thread_rng ().fill_bytes (&mut idem_id);
+		
+		Message::Request1 {
+			idem_id,
+			mac: None,
+		}
+	}
+}
+
 #[derive (Debug, PartialEq)]
 pub struct Response2 {
 	pub idem_id: [u8; 8],
 	pub nickname: String,
 }
 
-#[derive (Debug, Error)]
+#[derive (Debug, thiserror::Error)]
 pub enum MessageError {
 	#[error (transparent)]
 	Io (#[from] std::io::Error),
@@ -66,7 +69,7 @@ impl Write for DummyWriter {
 }
 
 impl Message {
-	pub fn write <T> (&self, w: &mut Cursor <T>) -> Result <(), std::io::Error> 
+	pub fn write <T> (&self, w: &mut Cursor <T>) -> Result <(), MessageError> 
 	where Cursor <T>: Write
 	{
 		match self {
@@ -92,7 +95,7 @@ impl Message {
 				Self::write_response_2 (&mut dummy_writer, x)?;
 				
 				// Write length and real params to real output
-				let len = u32::try_from (dummy_writer.position).unwrap ();
+				let len = u32::try_from (dummy_writer.position)?;
 				w.write_all (&len.to_le_bytes ())?;
 				Self::write_response_2 (w, x)?;
 			},
@@ -102,13 +105,11 @@ impl Message {
 	}
 	
 	fn write_response_2 <W: Write> (w: &mut W, params: &Response2) 
-	-> Result <(), std::io::Error>
+	-> Result <(), MessageError>
 	{
 		w.write_all (&params.idem_id)?;
 		let nickname = params.nickname.as_bytes ();
-		let nickname_len = u32::try_from (nickname.len ()).unwrap ();
-		w.write_all (&nickname_len.to_le_bytes ())?;
-		w.write_all (&nickname)?;
+		tlv::Writer::<_>::lv_bytes (w, nickname)?;
 		Ok (())
 	}
 	
@@ -124,14 +125,14 @@ impl Message {
 		Ok (())
 	}
 	
-	pub fn to_vec (&self) -> Result <Vec <u8>, tlv::TlvError> {
+	pub fn to_vec (&self) -> Result <Vec <u8>, MessageError> {
 		let mut cursor = Cursor::new (Vec::with_capacity (PACKET_SIZE));
 		cursor.write_all (&MAGIC_NUMBER)?;
 		self.write (&mut cursor)?;
 		Ok (cursor.into_inner ())
 	}
 	
-	pub fn many_to_vec (msgs: &[Self]) -> Result <Vec <u8>, tlv::TlvError> {
+	pub fn many_to_vec (msgs: &[Self]) -> Result <Vec <u8>, MessageError> {
 		let mut cursor = Cursor::new (Vec::with_capacity (PACKET_SIZE));
 		cursor.write_all (&MAGIC_NUMBER)?;
 		for msg in msgs {
@@ -140,28 +141,6 @@ impl Message {
 		Ok (cursor.into_inner ())
 	}
 	
-	fn read1 <R: std::io::Read> (r: &mut R) -> Result <Self, MessageError> {
-		let t = tlv::Reader::u8 (r)?;
-		
-		Ok (match t {
-			1 => {
-				let mut idem_id = [0u8; 8];
-				r.read_exact (&mut idem_id)?;
-				
-				let mac = Self::read_mac_opt (r)?;
-				Self::Request1 {
-					idem_id,
-					mac,
-				}
-			},
-			2 => {
-				let mac = Self::read_mac_opt (r)?;
-				Self::Response1 (mac)
-			},
-			_ => return Err (MessageError::UnknownType),
-		})
-	}
-	
 	fn read2 <R: std::io::Read> (r: &mut R) -> Result <Self, MessageError> {
 		let t = tlv::Reader::u8 (r)?;
 		
@@ -181,24 +160,12 @@ impl Message {
 				Self::Response1 (mac)
 			},
 			3 => {
-				let mut len = [0; 4];
-				r.read_exact (&mut len)?;
-				let _len = len;
+				tlv::Reader::<_>::length (r)?;
 				
 				let mut idem_id = [0; 8];
 				r.read_exact (&mut idem_id)?;
 				
-				let mut nickname_len = [0; 4];
-				r.read_exact (&mut nickname_len)?;
-				let nickname_len = u32::from_le_bytes (nickname_len);
-				let nickname_len = usize::try_from (nickname_len)?;
-				
-				if nickname_len > 64 {
-					return Err (MessageError::LengthPrefixTooLong ((64, nickname_len)));
-				}
-				
-				let mut nickname = vec! [0u8; nickname_len];
-				r.read_exact (&mut nickname)?;
+				let nickname = tlv::Reader::<_>::lv_bytes_to_vec (r, 64)?;
 				let nickname = String::from_utf8 (nickname)?;
 				
 				Self::Response2 (Response2 {
@@ -223,12 +190,6 @@ impl Message {
 		})
 	}
 	
-	pub fn from_slice1 (buf: &[u8]) -> Result <Self, MessageError> {
-		let mut cursor = Cursor::new (buf);
-		tlv::Reader::expect (&mut cursor, &MAGIC_NUMBER)?;
-		Self::read1 (&mut cursor)
-	}
-	
 	pub fn from_slice2 (buf: &[u8]) -> Result <Vec <Self>, MessageError> {
 		let mut cursor = Cursor::new (buf);
 		tlv::Reader::expect (&mut cursor, &MAGIC_NUMBER)?;
@@ -248,7 +209,7 @@ mod test {
 	use super::*;
 	
 	#[test]
-	fn test_write_2 () {
+	fn test_write_2 () -> Result <(), MessageError> {
 		for (input, expected) in [
 			(
 				vec! [
@@ -296,13 +257,15 @@ mod test {
 				],
 			),
 		] { 
-			let actual = Message::many_to_vec (&input).unwrap ();
+			let actual = Message::many_to_vec (&input)?;
 			assert_eq! (actual, expected, "{:?}", input);
 		}
+		
+		Ok (())
 	}
 	
 	#[test]
-	fn test_write_1 () {
+	fn test_write_1 () -> Result <(), MessageError> {
 		for (input, expected) in [
 			(
 				Message::Request1 {
@@ -344,13 +307,15 @@ mod test {
 				],
 			),
 		].into_iter () {
-			let actual = input.to_vec ().unwrap ();
+			let actual = input.to_vec ()?;
 			assert_eq! (actual, expected, "{:?}", input);
 		}
+		
+		Ok (())
 	}
 	
 	#[test]
-	fn test_read_2 () {
+	fn test_read_2 () -> Result <(), MessageError> {
 		for input in [
 			vec! [
 				Message::Request1 {
@@ -372,83 +337,11 @@ mod test {
 				}),
 			],
 		].into_iter () {
-			let encoded = Message::many_to_vec (&input).unwrap ();
-			let decoded = Message::from_slice2 (&encoded).unwrap ();
-			assert_eq! (input, decoded);
-		}
-	}
-	
-	#[test]
-	fn test_read_1 () {
-		for input in [
-			Message::Request1 {
-				idem_id: [0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08],
-				mac: None,
-			},
-			Message::Response1 (Some ([0x11, 0x22, 0x33, 0x44, 0x55, 0x66])),
-			Message::Response1 (None),
-		].into_iter () {
-			let encoded = input.to_vec ().unwrap ();
-			let decoded = Message::from_slice1 (&encoded).unwrap ();
+			let encoded = Message::many_to_vec (&input)?;
+			let decoded = Message::from_slice2 (&encoded)?;
 			assert_eq! (input, decoded);
 		}
 		
-		for (expected, input) in [
-			(
-				Message::Request1 {
-					idem_id: [0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08],
-					mac: None,
-				},
-				vec! [
-					154, 74, 67, 129,
-					// Request tag
-					1,
-					// Idem ID
-					1, 2, 3, 4, 5, 6, 7, 8,
-					// MAC is None
-					0,
-				],
-			),
-			(
-				Message::Response1 (Some ([0x11, 0x22, 0x33, 0x44, 0x55, 0x66])), 
-				vec! [
-					// Magic number for LookAround packets
-					154, 74, 67, 129,
-					// Response tag
-					2, 
-					// MAC is Some
-					1,
-					// MAC
-					17, 34, 51, 68, 85, 102,
-				],
-			),
-			(
-				Message::Response1 (None), 
-				vec! [
-					// Magic number for LookAround packets
-					154, 74, 67, 129,
-					// Response tag
-					2, 
-					// MAC is None
-					0,
-				],
-			),
-			(
-				Message::Response1 (None), 
-				vec! [
-					// Magic number for LookAround packets
-					154, 74, 67, 129,
-					// Response tag
-					2, 
-					// MAC is None
-					0,
-					// New tag that older versions will just ignore
-					255,
-				],
-			),
-		].into_iter () {
-			let actual = Message::from_slice1 (&input).unwrap ();
-			assert_eq! (actual, expected, "{:?}", actual);
-		}
+		Ok (())
 	}
 }
